# -*- coding: utf-8 -*-

import re
import codecs
import csv
import io
import json
from urllib.parse import urlencode

from odoo import http, fields
from odoo.http import request, content_disposition
from odoo.tools import str2bool
from odoo.tools.misc import xlsxwriter

from odoo.addons.l10n_be_hr_contract_salary.controllers.main import SignContract, HrContractSalary

class SignContractInternal(SignContract):

    def _update_contract_on_signature(self, request_item, contract, offer):
        super()._update_contract_on_signature(request_item, contract, offer)
        # Both applicant/employee and HR responsible have signed
        if request_item.sign_request_id.nb_closed == 2:
            if contract.employee_id and contract.job_id.id == 1:  # Python & JS Developer
                training_department = request.env['hr.department'].browse(197)  # RD Training
                if training_department.exists():
                    if not contract.department_id:
                        contract.department_id = training_department
                    if not contract.employee_id.department_id:
                        contract.employee_id.department_id = training_department
            if contract.terminal_id and contract.terminal_id.driver_id != contract.employee_id.work_contact_id:
                contract.terminal_id.driver_id = contract.employee_id.work_contact_id
            if contract.ordered_car_id and not contract.applicant_id:
                contract.ordered_car_id.state_id = request.env.ref('fleet.fleet_vehicle_state_waiting_list')


class WebsiteHrContractSalary(HrContractSalary):

    @http.route(['/salary_package/simulation/contract/<int:contract_id>'], type='http', auth="public", website=True, sitemap=False)
    def salary_package_deprecated(self, contract_id=None, **kw):
        # we should only limit this controller to public contract used for salaray configurator
        # (and enforce presence of `job_id` in request args)
        contract = request.env["hr.contract"].sudo().search([
            ('id', '=', contract_id),
            ('employee_id', '=', False),
        ])
        if not contract or not kw.get('job_id'):
            return super().salary_package_deprecated()  # outdated link

        job_id = int(kw['job_id'])
        final_yearly_costs = float(kw.get('final_yearly_costs') or 0.0)
        department_id = contract.department_id.id

        # look for a matching public offer
        offer = request.env['hr.contract.salary.offer'].sudo().search([
            ('contract_template_id', '=', contract.id),
            ('employee_job_id', '=', job_id),
            ('employee_id', '=', False),
            ('applicant_id', '=', False),
            ('department_id', '=', department_id),
            ('final_yearly_costs', '=', final_yearly_costs),
            ('state', '=', 'open'),
        ], order='id DESC', limit=1)
        if not offer:
            # or create a new one
            salary_simulation = request.env["generate.simulation.link"].sudo().new({
                'contract_id': contract_id,
                'final_yearly_costs': float(kw['final_yearly_costs'] or 0.0),
                'employee_job_id': job_id,
                'department_id': department_id,
                'new_car': str2bool(kw.get('new_car', '0')),
            })
            offer_values = salary_simulation._get_offer_values()
            # longer expiry for public simulation
            offer_values['offer_end_date'] = fields.Date.add(fields.Date.today(), months=3)
            offer = request.env['hr.contract.salary.offer'].sudo().with_context(
                tracking_disable=True,
            ).create(offer_values)

        simulation_link = "/salary_package/simulation/offer/%d?%s" % (
            offer.id, urlencode({'job_id': job_id}),
        )
        return request.redirect(simulation_link)

    def _get_default_template_values(self, contract, offer):
        values = super()._get_default_template_values(contract, offer)
        if not offer.applicant_id and not offer.employee_contract_id:
            values.update(is_simulation=True,
                          redirect_to_job=str(offer.employee_job_id.id))
        if contract.bike_id.log_contracts.filtered(lambda l: l.state == 'open'):
            values['disable_bike_selection'] = True

        terminal = offer.terminal_id or contract.terminal_id
        if terminal.log_contracts.filtered(lambda l: l.state == 'open'):
            values['disable_terminal_selection'] = True

        return values

    @http.route(["/export/insurance/"], type='http', auth='user', readonly=True)
    def export_insurance(self, contract_ids, **post):
        contracts = request.env['hr.contract'].browse(json.loads(contract_ids))
        contracts = contracts.filtered(lambda contract: contract.has_hospital_insurance)
        with io.BytesIO() as _buffer:
            _writer = codecs.getwriter('utf-8')
            writer = csv.writer(_writer(_buffer), quoting=csv.QUOTE_ALL)
            fields = "Employee Name,Employee Email,# Adults,# Children"
            writer.writerow(fields.split(","))
            writer.writerows([(
                contract.employee_id.name,
                contract.employee_id.private_email or contract.employee_id.work_email or 'Unknown',
                contract.insured_relative_adults_total,
                contract.insured_relative_children,
            ) for contract in contracts])
            csv_content = _buffer.getvalue()
        headers = [
            ('Content-Type', 'text/csv'),
            ('Content-Disposition', content_disposition('Insurance Data.csv'))
        ]
        return http.request.make_response(csv_content, headers=headers)

    @http.route(["/export/monizze/<int:batch_id>"], type='http', auth='user', readonly=True)
    def export_monizze(self, batch_id):
        batch = request.env['hr.payslip.run'].browse(batch_id)
        if not batch.exists() or not request.env.user.has_group('hr_payroll.group_hr_payroll_user'):
            return request.render(
                'http_routing.http_error', {
                    'status_code': 'Oops',
                    'status_message': 'This contract has been updated, please request an updated link..'})

        output = io.BytesIO()
        workbook = xlsxwriter.Workbook(output, {'in_memory': True})
        worksheet = workbook.add_worksheet('Worksheet')
        style_highlight = workbook.add_format({'bold': True, 'pattern': 1, 'bg_color': '#E0E0E0', 'align': 'center'})
        style_normal = workbook.add_format({'align': 'center'})
        row = 0

        headers = [
            "Numéro de registre national (p.ex. 790227 183 12)",
            "Salarié nom (p.ex. dupont)",
            "Salarié prénom (p.ex. max)",
            "Votre numéro interne du salarié  (p.ex. 152d97)",
            "Nombre de chèques [a] (p.ex. 18)",
            "Valeur faciale du chèque [b] (p.ex. 5.5)",
            "Total [a] x [b] (p.ex. 99)",
            "Date de naissance du salarié (dd/mm/yyyy)",
            "Sexe du salarié (m/f)",
            "Langue du salarié (nl/fr/en)",
            "Centre de coûts (p.ex. cc1. maximum 10 caractères)",
            "Votre numéro d'entreprise  (p.ex. be 0834013324)",
            "Adresse de livraison rue (p.ex. av. des volontaires)",
            "Adresse de livraison numéro (p.ex. 19)",
            "Adresse de livraison boite (p.ex. a1)",
            "Adresse de livraison code postal (p.ex. 1160)",
            "Adresse de livraison ville (p.ex. auderghem)",
        ]

        rows = []
        monthly_pay = request.env.ref('l10n_be_hr_payroll.hr_payroll_structure_cp200_employee_salary')
        struct_type_cp200 = request.env.ref('hr_contract.structure_type_employee_cp200')
        for payslip in batch.slip_ids:
            # Exclude students from meal vouchers to avoid creating new card for them
            if payslip.struct_id != monthly_pay or payslip.struct_id.type_id != struct_type_cp200:
                continue
            employee = payslip.employee_id
            employee_name = re.sub(r"[\(].*?[\)]", "", employee.name)
            quantity = payslip.line_ids.filtered(lambda l: l.code == 'MEAL_V_EMP').quantity
            amount = payslip.contract_id.meal_voucher_amount
            birthdate = employee.birthday or fields.Date.today()
            lang = employee.sudo().lang
            if lang == 'fr_FR':
                lang = 'FR'
            elif lang == 'nl_NL':
                lang = 'NL'
            else:
                lang = 'EN'

            rows.append((
                employee.niss.replace('.', '').replace('-', ''),
                employee_name.split(' ')[0],
                ' '.join(employee_name.split(' ')[1:]),
                ' ',
                quantity,
                amount,
                quantity * amount,
                '%s/%s/%s' % (birthdate.day, birthdate.month, birthdate.year),
                'F' if employee.gender == 'female' else 'M',
                lang,
                ' ', ' ', ' ', ' ', ' ', ' ', ' '
            ))

        col = 0
        for header in headers:
            worksheet.write(row, col, header, style_highlight)
            worksheet.set_column(col, col, 15)
            col += 1

        row = 1
        for employee_row in rows:
            col = 0
            for employee_data in employee_row:
                worksheet.write(row, col, employee_data, style_normal)
                col += 1
            row += 1

        workbook.close()
        xlsx_data = output.getvalue()
        response = request.make_response(
            xlsx_data,
            headers=[
                ('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'),
                ('Content-Disposition', 'attachment; filename=orderfile_odoo_SA.xlsx')],
        )
        return response

    def _get_benefits_values(self, contract, offer):
        advs, deps, req, req_name, types, dd_opts, dd_grp_opts, init_vals = super()._get_benefits_values(contract, offer)

        if offer.terminal_id:
            dd_opts['company_terminal_total_depreciated_cost'] = [(
                str(offer.terminal_id.id),
                '%s %s: %s€  (%s€/month for two years)' % (
                    offer.terminal_id.model_id.name,
                    offer.terminal_id.license_plate,
                    offer.terminal_id.car_value,
                    round(offer.terminal_id.total_depreciated_cost, 2),
                )
            )]
            init_vals['fold_company_terminal_total_depreciated_cost'] = True
            init_vals['company_terminal_total_depreciated_cost'] = offer.terminal_id.total_depreciated_cost
        else:
            init_vals['fold_company_terminal_total_depreciated_cost'] = False
            init_vals['company_terminal_total_depreciated_cost'] = 0

        return advs, deps, req, req_name, types, dd_opts, dd_grp_opts, init_vals
